import Canvas from '../references/canvas.js';


class ForegroundCanvas extends Canvas {
  Options = null;
  Debug = false;

  StartDate = new Date();
  Tasks = [];


  constructor(el) {
    super(el);
  }


  get HeaderHeight() {
    const a = this;

    return a.Options.HeaderRow.Height[0] + a.Options.HeaderRow.Height[1];
  }


  Load(startDate, tasks) {
    const a = this;

    a.StartDate = new Date(startDate);
    a.Tasks = tasks;
  }

  Invalidate() {
    const a = this;

    if (a.Options == null) {
      return;
    }

    if (a.StartDate == null) {
      return;
    }

    a.#drawTasks();
    a.#drawConnectorLines();
  }


  #drawTasks() {
    const a = this;

    for (let i=0; i<a.Tasks.length; i++) {
      let style = a.Options.Row.Task;

      if (a.Tasks[i].IsCollated == true) {
        style = a.Options.Row.CollatedTask;
      } else if (a.Tasks[i].CollatedTaskID != null) {
        style = a.Options.Row.ChildTask;
      } else if (a.Tasks[i].PredecessorTaskID == null) {
        style = a.Options.Row.OrphanTask;
      }

      const rectangle = a.#getTaskRectangle(a.Tasks[i], style);

      if (a.Tasks[i].Duration <= 0) {
        a.DrawDiamond(rectangle, style.BorderColour, { FillColour: style.FillColour });
      } else {
        a.DrawRectangle(rectangle, style.BorderColour, { FillColour: style.FillColour });
      }
    }
  }

  #drawConnectorLines() {
    const a = this;

    for (let i=0; i<a.Tasks.length; i++) {
      if (a.Tasks[i].PredecessorTaskID == null) {
        continue;
      }

      if (a.Tasks[i].PredecessorTaskID == null) {
        continue;
      }

      const style = ((a.Tasks[i].IsCollated == true) ? a.Options.Row.CollatedTask : a.Options.Row.Task);
      const rectangle = a.#getTaskRectangle(a.Tasks[i], style);
      const paddingX = (a.Options.Row.CollatedTask.Height + a.Options.BorderWidth);
      const offsetX = (rectangle.X + paddingX);

      const predecessorTask = a.Tasks.first("ID", a.Tasks[i].PredecessorTaskID);
      if (predecessorTask == null) {
        continue;
      }

      const predecessorStyle = ((predecessorTask.IsCollated == true) ? a.Options.Row.CollatedTask : a.Options.Row.Task);
      const predecessorRectangle = a.#getTaskRectangle(predecessorTask, predecessorStyle);

      let points = [];
      points.push({ X: (predecessorRectangle.X + predecessorRectangle.W), Y: (predecessorRectangle.Y + Math.half(predecessorRectangle.H)) });
      points.push({ X: offsetX, Y: (predecessorRectangle.Y + Math.half(predecessorRectangle.H)) });

      points.push({ X: offsetX, Y: (rectangle.Y - paddingX) });
      points.push({ X: offsetX, Y: rectangle.Y });

      a.DrawLines(points, a.Options.Line.Colour, { LineWidth: a.Options.Line.Width });

      const arrowRectangle = {
        X: (offsetX - Math.half(a.Options.Line.ArrowSize)),
        Y: (rectangle.Y - a.Options.Line.ArrowSize),
        W: a.Options.Line.ArrowSize,
        H: a.Options.Line.ArrowSize
      }

      a.DrawArrowS(arrowRectangle, a.Options.Line.Colour, { FillColour: a.Options.Line.Colour });
    }
  }

  #getTaskRectangle(task, style) {
    const a = this;
    const i = (task.Order - 1);

    const rowRectangle = {
      X: 0,
      Y: (a.HeaderHeight + (a.Options.Row.Height * i)),
      W: a.Width,
      H: (a.Options.Row.Height - a.Options.BorderWidth)
    };

    if (a.Debug) {
      a.DrawRectangle(rowRectangle, "red", {});
    }

    if (task.Duration <= 0) {
      let result = {
        X: (a.Options.DayWidth * (Date.diffDays(a.StartDate, task.StartDate) + 1)),
        Y: (a.HeaderHeight + (a.Options.Row.Height * i) + Math.half(rowRectangle.H - style.Height)),
        W: style.Height,
        H: style.Height
      };

      result.X += Math.half(a.Options.DayWidth - (style.Height + a.Options.BorderWidth));

      return result;
    } else {
      return {
        X: (a.Options.DayWidth * (Date.diffDays(a.StartDate, task.StartDate) + 1)),
        Y: (a.HeaderHeight + (a.Options.Row.Height * i) + Math.half(rowRectangle.H - style.Height)),
        W: (a.Options.DayWidth * task.Duration),
        H: style.Height
      };
    }
  }



}


export default ForegroundCanvas;