class CanvasContainer {

  constructor(el) {
    const a = this;

    a.el = el;

    a.Layer = [];
    a.BorderWidth = 2;
    a.ScrollbarWidth = 11;
    a.Padding = {
      Top: 0,
      Right: 0,
      Bottom: 0,
      Left: 0
    };
    a.EnableDock = true;

    a.#initialiseComponents();
  }

  #initialiseComponents() {
    const a = this;

    a.ClearLayers();

    // a.Invalidate();
    // a.Clear();
  }


  get FlowContainer() {
    return this.el.getElementsByTagName("div")[0];
  }

  get Height() {
    const a = this;

    if (a.Layer.length <= 0) {
      return 0;
    }

    return Document.getHeight(a.Layer[0].el);
  }

  set Height(value) {
    const a = this;

    // if (a.el != null) {
    //   a.el.style.height = value + "px";
    // }

    if (a.FlowContainer != null) {
      a.FlowContainer.style.height = value + "px";
    }

    // if (a.CanvasContainer != null) {
    //   a.CanvasContainer.style.height = value + "px";
    // }

    // if (a.CanvasContext != null) {
    //   a.CanvasContext.canvas.height = value;
    // }

    a.Layer.forEach((e, i) => {
      e.Height = value;
    });
  }

  get Width() {
    const a = this;

    if (a.Layer.length <= 0) {
      return 0;
    }

    return Document.getWidth(a.Layer[0].el);
  }

  set Width(value) {
    const a = this;

    // if (a.el != null) {
    //   a.el.style.width = value + "px";
    // }

    if (a.FlowContainer != null) {
      a.FlowContainer.style.width = value + "px";
    }

    // if (a.CanvasContainer != null) {
    //   a.CanvasContainer.style.width = value + "px";
    // }

    // if (a.CanvasContext != null) {
    //   a.CanvasContext.canvas.width = value;
    // }

    a.Layer.forEach((e, i) => {
      e.Width = value;
    });
  }

  get Size() {
    return {
      W: this.Width,
      H: this.Height
    };
  }

  set Size(value) {
    this.Width = value.W;
    this.Height = value.H;
  }

  get ClientRectangle() {
    const a = this;

    return {
      X: a.Padding.Left,
      Y: a.Padding.Top,
      W: (a.Width - (a.Padding.Left + a.Padding.Right)),
      H: (a.Height - (a.Padding.Top + a.Padding.Bottom))
    };
  }

  get Rectangle() {
    const a = this;

    return {
      X: 0,
      Y: 0,
      W: a.Width,
      H: a.Height
    };
  }


  Clear() {
    const a = this;

    // if (a.CanvasContext == null) {
    //   return;
    // }

    // a.CanvasContext.clearRect(0, 0, a.CanvasContext.canvas.width, a.CanvasContext.canvas.height);

    a.Layer.forEach((e, i) => {
      e.Clear();
    });
  }

  ClearLayers() {
    const a = this;

    if (a.el != null) {
      a.el.innerHTML = "<div class='border'></div>";
    }

    a.Layer = [];

    a.Invalidate();
  }

  AddLayer() {
    const a = this;

    if (a.el == null) {
      return;
    }

    const divList = a.el.getElementsByTagName("div");
    if (divList.length <= 0) {
      return;
    }

    // Add another canvas element
    Document.appendHtml(divList[0], "canvas");

    const canvasList = a.el.getElementsByTagName("canvas");
    const result = canvasList[(canvasList.length - 1)];

    return result;
  }

  Invalidate() {
    const a = this;

    if (a.EnableDock) {
      a.FlowContainer.style.width = "100%";
    }

    const w = (a.Layer.length <= 0 ? 0 : Document.getWidth(a.Layer[0].el));
    const h = (a.Layer.length <= 0 ? 0 : Document.getHeight(a.Layer[0].el));

    if (w > (Document.getWidth(a.FlowContainer) + (a.BorderWidth * 2))) {
      a.FlowContainer.classList.add("scroll-x");
      a.FlowContainer.style.height = (h + a.ScrollbarWidth) + "px";
    } else {
      a.FlowContainer.classList.remove("scroll-x");
      a.FlowContainer.style.height = h + "px";
    }

    a.Layer.forEach(e => {
      if (typeof(e.Invalidate) != "undefined") {
        e.Invalidate();
      }
    });

    a.Layer.forEach((e, i) => {
      if (i <= 0) {
        return;
      }

      // Correct positon because of position-relative to keep container overflow working
      const h = Document.getHeight(a.Layer[(i - 1)].el);

      a.Layer[i].el.style.top = "-" + h + "px";
    });
  }

}


export default CanvasContainer;